# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2005-2018, PyInstaller Development Team.
#
# Distributed under the terms of the GNU General Public License with exception
# for distributing bootloader.
#
# The full license is in the file COPYING.txt, distributed with this software.
#-----------------------------------------------------------------------------

'''
Mock module of the same name as _and_ shadowed by a global variable defined by
the `__init__` submodule of this package.

This module is exercised by the `test_import_submodule_global_shadowed`
functional test.
'''
