@import url("theme.css");

/* tune option lists */
.rst-content table.docutils.option-list tr td {
    white-space: normal !important;
}
.rst-content table.docutils.option-list tr:nth-child(2n-1) td {
    background-color: transparent !important;
}

.wy-nav-side { background-color: white ; }


tt, span.literal {
	font-family: monospace;
	font-size: 0.95em;
	background-color: #ECF0F3;
	padding: 0 1px;
}

dl, table {
	margin-left: 2em;
	margin-right: 2em;
}

h1, h2, h3, h4, h5, h6 {
    background-color: #F2F2F2;
    border-bottom: 1px solid #CCCCCC;
    color: #20435C !important;
    font-family: 'Trebuchet MS',sans-serif;
    font-weight: normal;
    margin: 20px -20px 10px;
    padding: 3px 0 3px 10px;
}

h1 {
	text-align: center;
	font-variant: small-caps;
}

a:link, a:visited { color: #006699; }
a:hover { background-color:#006699; color:white; }

h1 a, h2 a, h3 a, h4 a, h5 a, h6 a,
h1 a:hover, h2 a:hover, h3 a:hover, h4 a:hover, h5 a:hover, h6 a:hover,
h1 a:visited, h2 a:visited, h3 a:visited, h4 a:visited, h5 a:visited, h6 a:visited {
	text-decoration: inherit;
	background-color: inherit;
	color:inherit;
}
h2 a:hover::after, h3 a:hover::after {
	color: red;
	content: " back to contents";
	text-decoration: unterline;
	font-size: x-small;
	margin-left: 1em;
}

pre {
  background-color: #EEFFCC;
  border-color: #AACC99;
  border-style: solid none;
  border-width: 1px medium;
  color: #333333;
  line-height: 120%;
  padding: 0.5em;
  margin-left: 4em;
  margin-right: 4em;
  font-family: monospace;
  font-size: 0.95em;
}

code, .rst-content tt, .rst-content code {
    background-color: #ECF0F3;
}

a.reference.commitid {
	font-size: 95%;
}

// Admonitions

div.note p.admonition-title {
    font-weight: bold;
    float: left;
    margin-right: 1em;
}

div.note p.admonition-title:after {
    content: ": "
}
