#-----------------------------------------------------------------------------
# Copyright (c) 2005-2018, PyInstaller Development Team.
#
# Distributed under the terms of the GNU General Public License with exception
# for distributing bootloader.
#
# The full license is in the file COPYING.txt, distributed with this software.
#-----------------------------------------------------------------------------

"""
This is a module whose name begins with 're'. It will be used to verify that the
regex for excluding modules listed in PY3_BASE_MODULES (which includes 're') does
not exclude this module.
"""